/*
//
//       Elvees Performance Platform
//             core functions
//
*/

#pragma once

#include "eppdefs.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
//  Name:      epp_GetLibraryVersion
//  Purpose:   getting of the library version
//  Parameters:
//  Returns:   the structure of information about version of eppcore library
//  Notes:     don't free the pointer
*/
EPPAPI( const epp_LibraryVersion*, epp_GetLibraryVersion, (void) )


/*
//  Name:      epp_GetStatusString
//  Purpose:   convert the library status code to a readable string
//  Parameters:
//    sts      EPP status
//  Returns:   pointer to string describing the library status code
//  Notes:     don't free the pointer
*/
EPPAPI( const char*, epp_GetStatusString, (epp_Status sts) )


/*
//  Name:      epp_InitExecContext
//  Purpose:   init global execution context
//  Returns:   epp_Status
//    epp_StsOK           Ok
//    epp_StsMemAllocErr  Not enough memory allocated for the operation
//  Parameters:
//  Notes:
*/
EPPAPI( epp_Status, epp_InitExecContext, (void) )


/*
//  Name:      epp_GetExecContext
//  Purpose:   get current global execution context
//  Returns:   epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pContext  The pointer to the execution context
//  Notes:
*/
EPPAPI( epp_Status, epp_GetExecContext, (epp_ExecContext *pContext) )


/*
//  Name:      epp_SetExecContext
//  Purpose:   set current global execution context
//  Returns:   epp_Status
//    epp_StsOK              Ok
//    epp_StsOutOfRangeErr   resources in context are out of range
//  Parameters:
//    pContext  The pointer to the execution context
//  Notes:
*/
EPPAPI( epp_Status, epp_SetExecContext, (const epp_ExecContext *pContext) )


/*
//  Name:      epp_SetExecContextMode
//  Purpose:   set mode of current global execution context
//  Returns:   epp_Status
//    epp_StsOK                  Ok
//    epp_StsExecContextModeErr  Unsupported mode of execution context
//  Parameters:
//    pContext  The pointer to the execution context
//  Notes:
*/
EPPAPI( epp_Status, epp_SetExecContextMode, (epp_ExecMode mode) )


/*
//  Name:      epp_GetCpuInfo
//  Purpose:   gets cpu info
//  Parameters:
//      pInfo  pointer to the epp_CpuInfo
//  Returns:
//  Notes:
*/
EPPAPI( void, epp_GetCpuInfo, (epp_CpuInfo* pInfo) )


/*
//  Name:      epp_GetCpuTickCounter
//  Purpose:   reading of cpu tick counter value
//  Parameters:
//  Returns:   tick counter value
//  Notes:
*/
EPPAPI( epp_u64, epp_GetCpuClockCounter, (void) )


/*
//  Name:      epp_GetCpuInstructionCounter
//  Purpose:   reading of cpu instruction counter value
//  Parameters:
//  Returns:   instruction counter value
//  Notes:
*/
EPPAPI( epp_u64, epp_GetCpuInstructionCounter, (void) )


/*
//  Name:      epp_GetL2DataCacheSize
//  Purpose:   get current size of L2D cache
//  Returns:
//    epp_L2DataCacheSize  The size of L2D cache
//  Parameters:
//  Notes:
*/
EPPAPI( epp_L2DataCacheSize, epp_GetL2DataCacheSize, (void) )


/*
//  Name:      epp_SetL2DataCacheSize
//  Purpose:   set size of L2D cache
//  Returns:   epp_Status
//    epp_StsOK                  Ok
//    epp_StsL2DataCacheSizeErr  Unsupported size of L2D cache
//  Parameters:
//    size     The size of L2D cache
//  Notes:
*/
EPPAPI( epp_Status, epp_SetL2DataCacheSize, (epp_L2DataCacheSize size) )


/*
//  Name:      epp_GetL1DataCachePFVN
//  Purpose:   get current prefetch size for vector transaction of L1D cache
//  Returns:
//    epp_L1DataCachePFN  The prefetch size
//  Parameters:
//  Notes:
*/
EPPAPI( epp_L1DataCachePFN, epp_GetL1DataCachePFVN, (void) )


/*
//  Name:      epp_SetL1DataCachePFVN
//  Purpose:   set prefetch size for vector transaction of L1D cache
//  Returns:   epp_Status
//    epp_StsOK                  Ok
//    epp_StsL1DataCachePFNErr   Unsupported size of L2D cache prefetch lines
//  Parameters:
//    size     The number of prefetch lines of L1D cache
//  Notes:
*/
EPPAPI( epp_Status, epp_SetL1DataCachePFVN, (epp_L1DataCachePFN size) )


/*
//  Name:      epp_FlushDataCaches
//  Purpose:   flushes all data caches
//  Parameters:
//  Returns:
//  Notes:
*/
EPPAPI( void, epp_FlushDataCaches, (void) )


/*
//  Name:      epp_FlushDataCachesPtr
//  Purpose:   flushes all data caches for memory block
//  Parameters:
//  Returns:
//  Notes:
*/
EPPAPI( void, epp_FlushDataCachesPtr, (void *ptr, epp_size size) )


/*
//  Name:      epp_FlushInvalidateDataCaches
//  Purpose:   flushes and invalidates all data caches
//  Parameters:
//  Returns:
//  Notes:
*/
EPPAPI( void, epp_FlushInvalidateDataCaches, (void) )


/*
//  Name:      epp_FlushInvalidateDataCachesPtr
//  Purpose:   flushes and invalidates all data caches for memory block
//  Parameters:
//  Returns:
//  Notes:
*/
EPPAPI( void, epp_FlushInvalidateDataCachesPtr, (void *ptr, epp_size size) )


/*
//  Name:      epp_InvalidateDataCachesPtr
//  Purpose:   invalidates all data caches for memory block
//  Parameters:
//  Returns:
//  Notes:
*/
EPPAPI( void, epp_InvalidateDataCachesPtr, (void *ptr, epp_size size) )


/*
//  Name:      epp_GetInternalMemoryPtr
//  Purpose:   get pointer to XYRAM
//  Returns:
//    void*    The pointer to XYRAM
//  Parameters:
//  Notes:
*/
EPPAPI( void*, epp_GetInternalMemoryPtr, (void) )


/*
//  Name:      epp_GetInternalMemorySize
//  Purpose:   get size of XYRAM
//  Returns:
//    epp_size  The size of XYRAM
//  Parameters:
//  Notes:
*/
EPPAPI( epp_size, epp_GetInternalMemorySize, (void) )


/*
//  Name:      epp_Malloc
//  Purpose:   aligned (by EPP_ALIGNMENT) memory allocation
//  Parameter:
//    size     number of bytes
//  Returns:   pointer to allocated memory
//  Notes:     the memory allocated by epp_Malloc has to be free by epp_Free function only.
*/

EPPAPI( void*, epp_Malloc, (epp_size size) )

EPPINL( epp_u8*, epp_Malloc_u8, (epp_size length))
{
    return (epp_u8*)epp_Malloc(length * sizeof(epp_u8));
}

EPPINL( epp_s8*, epp_Malloc_s8, (epp_size length))
{
    return (epp_s8*)epp_Malloc(length * sizeof(epp_s8));
}

EPPINL( epp_u16*, epp_Malloc_u16, (epp_size length))
{
    return (epp_u16*)epp_Malloc(length * sizeof(epp_u16));
}

EPPINL( epp_s16*, epp_Malloc_s16, (epp_size length))
{
    return (epp_s16*)epp_Malloc(length * sizeof(epp_s16));
}

EPPINL( epp_u32*, epp_Malloc_u32, (epp_size length))
{
    return (epp_u32*)epp_Malloc(length * sizeof(epp_u32));
}

EPPINL( epp_s32*, epp_Malloc_s32, (epp_size length))
{
    return (epp_s32*)epp_Malloc(length * sizeof(epp_s32));
}

EPPINL( epp_u64*, epp_Malloc_u64, (epp_size length))
{
    return (epp_u64*)epp_Malloc(length * sizeof(epp_u64));
}

EPPINL( epp_s64*, epp_Malloc_s64, (epp_size length))
{
    return (epp_s64*)epp_Malloc(length * sizeof(epp_s64));
}

EPPINL( epp_f16*, epp_Malloc_f16, (epp_size length))
{
    return (epp_f16*)epp_Malloc(length * sizeof(epp_f16));
}

EPPINL( epp_f32*, epp_Malloc_f32, (epp_size length))
{
    return (epp_f32*)epp_Malloc(length * sizeof(epp_f32));
}

EPPINL( epp_f64*, epp_Malloc_f64, (epp_size length))
{
    return (epp_f64*)epp_Malloc(length * sizeof(epp_f64));
}

EPPINL( epp_cs16*, epp_Malloc_cs16, (epp_size length))
{
    return (epp_cs16*)epp_Malloc(length * sizeof(epp_cs16));
}

EPPINL( epp_cs32*, epp_Malloc_cs32, (epp_size length))
{
    return (epp_cs32*)epp_Malloc(length * sizeof(epp_cs32));
}

EPPINL( epp_cs64*, epp_Malloc_cs64, (epp_size length))
{
    return (epp_cs64*)epp_Malloc(length * sizeof(epp_cs64));
}

EPPINL( epp_cf16*, epp_Malloc_cf16, (epp_size length))
{
    return (epp_cf16*)epp_Malloc(length * sizeof(epp_cf16));
}

EPPINL( epp_cf32*, epp_Malloc_cf32, (epp_size length))
{
    return (epp_cf32*)epp_Malloc(length * sizeof(epp_cf32));
}

EPPINL( epp_cf64*, epp_Malloc_cf64, (epp_size length))
{
    return (epp_cf64*)epp_Malloc(length * sizeof(epp_cf64));
}


/*
//  Name:      epp_Free
//  Purpose:   free memory allocated by the epp_Malloc function
//  Parameter:
//    ptr      pointer to the memory allocated by the epp_Malloc function
//  Notes:     use the function to free memory allocated by epp_Malloc
*/
EPPAPI( void, epp_Free, (void* ptr) )


/*
//  Name:      epp_f16f32
//  Purpose:   convert epp_f16 to epp_f32
//  Parameter:
//    val      epp_f16 value
//  Returns:   epp_f32 value
*/

EPPAPI( epp_f32, epp_f16f32,  (epp_f16 val) )


/*
//  Name:      epp_f32f16
//  Purpose:   convert epp_f32 to epp_f16
//  Parameter:
//    val      epp_f32 value
//  Returns:   epp_f16 value
//  Notes:     if val is out of range of epp_f16, truncation will be applied
*/

EPPAPI( epp_f16, epp_f32f16,  (epp_f32 val) )


#ifdef __cplusplus
}
#endif
